############################################################################
#    Copyright (C) 2009 by Kushal Das                                      #
#    kushaldas@gmail.com                                                   #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import flickr
import flickrapi
import os
import urllib2
from xml.dom.minidom import parseString
from PyQt4 import QtCore


flickr.API_KEY = "7414f54455bb6e61a560bad947d660d8"
flickr.API_SECRET = "4a56d4c5e9b2b0cd"


class UkhraFlickr(QtCore.QThread):
    """
    This is main class to work with Flickr
    I used both flickr.py and python-flickrapi here.

    Usage:
        >> from UkhraFlickr import *
        >>> u = UkhraFlickr()
        >>> res = u.gettoken()

    Now if the res == False then you have to ask the user to authenticate 
    using firefox and then get token from web.
       >>> u.openfirefox()
       >>> u.gettokenfromweb()

    If res == True that means authentication is done.

    To upload an image:
    It uses the say function as callback

    >>> def say(p,done):
    ...   if done:
    ...     print "done"
    ...   else:
    ...     print "%s%%" % p
    ...


    >>> u.upload('path/to/file',title="My Title",description="some desc",tags='tag1 tag2 "many tags"',callback=say)

    """

    def __init__(self,parent = None):
        QtCore.QThread.__init__(self, parent)
        self.myAuth = flickr.Auth()
        self.myflickr = flickrapi.FlickrAPI(flickr.API_KEY, flickr.API_SECRET)
        self.frob = self.myAuth.getFrob()
        self.link = self.myAuth.loginLink("write",self.frob)
        self.token = ""
        self.lastuploadedphotoid = None

    def openfirefox(self):
        firefox=os.popen( 'firefox \"' + self.link + '\"' )

    def gettoken(self):
        """
        This will get the token and download it to
        disk.
        """
        token_cache = flickrapi.tokencache.TokenCache(flickr.API_KEY)
        self.token = token_cache._TokenCache__get_cached_token()
        if not self.token:
            return False
        else:
            token_cache._TokenCache__set_cached_token(self.token)
            flickr.TOKEN = self.token
            self.myflickr.token_cache = token_cache
            return True

    def gettokenfromweb(self):
        self.token = self.myAuth.getToken(self.frob)
        flickr.TOKEN = self.token
        token_cache = flickrapi.tokencache.TokenCache(flickr.API_KEY)
        token_cache._TokenCache__set_cached_token(self.token)
        self.myflickr.token_cache = token_cache

    def getSets(self):
        user = flickr.test_login()
        return user.getPhotosets()


    def setImages(self, images):
        self.images = images

    def run(self):
        images = [(x,x.path) for x in self.images]
        for flickricon,path in images:
            self.upload(flickricon.path, \
                    flickricon.title,\
                    flickricon.description,\
                    flickricon.tags, \
                    self.updateFileProgressBar)
            #One image is done, now remove it from view
            self.emit(QtCore.SIGNAL("fileDone(PyQt_PyObject)"), flickricon)
        self.emit(QtCore.SIGNAL("uploadDone()"))
        return

    def updateFileProgressBar(self, progress, done):
        self.emit(QtCore.SIGNAL("updateProgressBar(PyQt_PyObject, PyQt_PyObject)"), progress, done)


    def upload(self, path, title="", description="", tags="", callback=None):
        """
        To upload pics to flickr
        Arguments:
            path:  path to the image file
           title:  title of the image
     description:  description of the image
            tags:  tags for the image
    callback:  the callback function
        """
        tags += ' pony'
        res = self.myflickr.upload(path, title=title,\
                             description = description, \
                             tags = tags, callback=callback,\
                             format="rest")
        dom = parseString(res)
        self.lastuploadedphotoid = dom.getElementsByTagName("photoid")[0].childNodes[0].data
